class NumSeq
	include Enumerable	
	def self.from_array(array, index_origin = 0)
		len = array.length
		seq = Sequence.new(len, index_origin) do |n, i|
			array[i]
		end
		return seq	
	end


	class SequenceArray < Array
		def initialize(n, offset)
			@offset = offset
			super(n)
		end
	
		def offset=(new_offset)
			raise "Wrong point reached" if new_offset >= @offset
			(@offset-new_offset).times { self.unshift nil }
			@offset = new_offset
		end
		
		def [](n)
			super(n - @offset)
		end

		def []=(n,value)
			super(n - @offset, value)
		end
	end

	def initialize(len, start_at = 0)
		if len < 0 then 
			raise ArgumentError, 
				"Length of a num_seq must be positive"
		end
	
		@@strict = false
		@undef_value = nil

		if 0 == len then
			@undef_inf = @undef_sup = start_at
		else
			@undef_inf = start_at - 1
			@undef_sup = start_at + len	
		end
		

		@elems = SequenceArray.new(len, @undef_inf + 1)

		if block_given? then
			((@undef_inf+1)...@undef_sup).each_with_index do |n,i|
				@elems[n] = yield(n,i)
			end		
		end
	end

	attr_accessor :undef_value

	def self.behave_strict?
		return @@strict
	end

	def self.strict_behavior
		@@strict = true
	end

	def self.rough_behavior
		@@strict = false
	end

	def seq_defined_at?(n)
		return n > @undef_inf && n < @undef_sup
	end
	private :seq_defined_at?

	def [](n)
		return @undef_value unless seq_defined_at? n 
		return @elems[n]
	end

	def []=(n, value)
		if self.class.behave_strict? and not seq_defined_at?(n) then
			raise IndexError,
				%Q%Attempt to assign at index #{n} % +
				%q%where the num_seq is not defined%
		end
		return @elems[n] = value if seq_defined_at? n
		if n <= @undef_inf then
			@undef_inf = n - 1
			@elems.offset = n
			@elems[n] = value 
		elsif n >= @undef_sup then
			@undef_sup = n + 1
			@elems[n] = value
		else
			raise "Impossible  point reached"
		end
		
	end

	def range
		return (@undef_inf+1..@undef_sup-1)
	end

	def length
		return @undef_sup - @undef_inf - 1
	end
	alias :size :length

	def min_index
		return @undef_inf + 1
	end
	alias :n_min :min_index

	def max_index
		return @undef_sup - 1
	end
	alias :n_max :max_index

	def first
		return self[n_min]
	end

	def last
		return self[n_max]
	end

	# y[n] = x[n-k]
	def shift(k)
		if k < 0 then
		# left shift
			_lshift(-k)
		elsif k > 0 then
		# right shift
			_rshift(k)
		else
		# return copy -> CHANGE!!
			return nil
		end
	end

	def rshift(k)
		if k < 0 then
			raise ArgumentError, "Argument should be non-negative"
		end
		return _rshift(k)
	end

	def lshift(k)
		if k < 0 then
			raise ArgumentError, "Argument should be non-negative"
		end
		return _lshift(k)
	end

	def _rshift(k)
		start_at = self.n_min
		length = self.length
		seq = Sequence.new(length, start_at + k) do |n|
			self[n-k]
		end
		return seq
	end
	private :_rshift
	
	def _lshift(k)
		start_at = self.n_min
		length = self.length
		seq = Sequence.new(length, start_at - k) do |n|
			self[n+k]
		end
		return seq
	end
	private :_lshift

	# x[Mn]
	def compress(m)
	end

	# x[n/L]
	def expand(l)
	end

	def to_s
		index = self.n_min
		@elems.each do |e|
			puts "#{index} #{e}"
			index += 1
		end	
	end
end

require 'num_seq/common'
require 'num_seq/enumerable'
require 'num_seq/array'
require 'num_seq/ops'
require 'num_seq/internal_ops/common'
require 'num_seq/internal_ops/sum'
require 'num_seq/internal_ops/sub'
require 'num_seq/internal_ops/conv'
require 'num_seq/internal_ops/mul'
require 'num_seq/internal_ops/div'
require 'num_seq/internal_ops/exp'
require 'num_seq/internal_ops/mod'
require 'num_seq/internal_ops/math'
require 'num_seq/internal_ops/interpolation'
require 'num_seq/external_ops/common'
require 'num_seq/external_ops/sum'
require 'num_seq/external_ops/sub'
require 'num_seq/external_ops/mul'
require 'num_seq/external_ops/div'
require 'num_seq/external_ops/mod'
require 'num_seq/external_ops/exp'
require 'num_seq/external_ops/energetic'
require 'num_seq/external_ops/stats'



